/***************************************************************************
 $RCSfile: kbanking.h,v $
                             -------------------
    cvs         : $Id: kbanking.h,v 1.18 2005/03/30 08:26:20 aquamaniac Exp $
    begin       : Mon Mar 01 2004
    copyright   : (C) 2004 by Martin Preuss
    email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/

#ifndef QBANKMANAGER_BANKING_H
#define QBANKMANAGER_BANKING_H

#include <aqbanking/banking.h>
#include <gwenhywfar/nettransportssl.h>

#include <qobject.h>
#include <list>

class KBanking;
class TransactionWindow;
class MainWindow;

#include "app.h"
#include "report.h"
#include "flagstaff.h"


class KBProgress;
class KBSimpleBox;

class KBanking: public App {
  friend class KBanking_Linker;
private:
  MainWindow *_mainWindow;
  GWEN_TYPE_UINT32 _lastWidgetId;
  AB_BANKING_LOGLEVEL _logLevel;
  std::list<KBProgress*> _progressWidgets;
  std::list<KBSimpleBox*> _simpleBoxWidgets;
  FlagStaff *_flagStaff;

  std::list<RefPointer<TransactionWindow> > _tWindows;

  /* variables from config file */
  std::list<Report*> _reports;
  unsigned int _lastReportId;

  /* private functions */
  KBProgress *_findProgressWidget(GWEN_TYPE_UINT32 id);
  void _cleanupProgressWidgets();
  static int _extractHTML(const char *text, GWEN_BUFFER *buf);
  void _sampleReportModules();

protected:
  virtual void infoTransactionsChanged(Account *a);

public:
  virtual int messageBox(GWEN_TYPE_UINT32 flags,
			 const char *title,
			 const char *text,
			 const char *b1,
			 const char *b2,
			 const char *b3);

  virtual int inputBox(GWEN_TYPE_UINT32 flags,
		       const char *title,
                       const char *text,
                       char *buffer,
                       int minLen,
                       int maxLen);

  virtual GWEN_TYPE_UINT32 showBox(GWEN_TYPE_UINT32 flags,
                                   const char *title,
                                   const char *text);
  virtual void hideBox(GWEN_TYPE_UINT32 id);

  virtual GWEN_TYPE_UINT32 progressStart(const char *title,
                                         const char *text,
                                         GWEN_TYPE_UINT32 total);

  /**
   * Advances the progress bar an application might present to the user.
   * @param id id assigned by @ref AB_Banking_Progress_Start
   * @param progress new value for progress. A special value is
   *  AB_BANKING_PROGRESS_NONE which means that the progress is unchanged.
   * This might be used as a keepalive call to a GUI.
   */
  virtual int progressAdvance(GWEN_TYPE_UINT32 id,
                              GWEN_TYPE_UINT32 progress);
  virtual int progressLog(GWEN_TYPE_UINT32 id,
                          AB_BANKING_LOGLEVEL level,
                          const char *text);
  virtual int progressEnd(GWEN_TYPE_UINT32 id);

  virtual int print(const char *docTitle,
                    const char *docType,
                    const char *descr,
                    const char *text);


public:
  KBanking(const char *appname,
           const char *fname=0,
           const char *name=0);
  virtual ~KBanking();


  int init();
  int fini();

  FlagStaff *flagStaff();

  virtual bool importContext(AB_IMEXPORTER_CONTEXT *ctx,
                             GWEN_TYPE_UINT32 flags);
  void setMainWindow(MainWindow *w);
  MainWindow * getMainWindow();

  bool setTabWidget(const QString &name);

  int enqueueJob(AB_JOB *j);
  int dequeueJob(AB_JOB *j);
  int executeQueue();

  void accountsUpdated();
  void transfersUpdated();
  void outboxCountChanged(int count);
  void statusMessage(const QString &s);

  void rebuildExportMenu();

  void invokeHelp(const char *subject);

  void addTransactionWindow(TransactionWindow *w);
  void delTransactionWindow(TransactionWindow *w);
  void delTransactionWindows();

  Report *findReport(const char *name);
  std::list<Report*> getReports();

  GWEN_DB_NODE *findReportProfile(Report *r, const char *id);
  GWEN_DB_NODE *getReportProfiles(Report *r);
  /** takes over ownership of the given group */
  int addReportProfile(Report *r, GWEN_DB_NODE *db);
  void removeReportProfile(Report *r,
                           GWEN_DB_NODE *dbProfile);

  static std::string QStringToUtf8String(const QString &qs);
  static std::string guiString(const char *s);

};




#endif /* QBANKMANAGER_BANKING_H */


